/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

/**
 **  test_simple_server.cpp
 **
 **  Small test server
 **/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gcspy_main_server.hpp"
#include "test_driver.hpp"

#ifdef _SOLARIS_
#include <thread.h>
#endif //_SOLARIS_

#ifdef _LINUX_
#include <pthread.h>
#endif //_LINUX_

#define MAX_LEN      ( 128 * 1024 )

#define EVENT_ONE     0
#define EVENT_TWO     1

gcspyMainServer *server;
testDriver *driver;

extern "C" {
static void *
main_loop (void *arg) {
  int counter = 0;

  server->wait_for_client();

  while ( 1 ) {
    gcspyTimer::wait(1000);
    int event = counter % 2;

    if (server->is_connected(event)) {
      server->start_compensation_timer();
      printf("Connected.\n");
      
      if (counter % 20 < 10)
	driver->reset(100);
      else
	driver->reset(150);

      driver->object(2);
      driver->object(2);
      driver->object(5);
      driver->object(1);
      driver->card(0, testDriver::card_state_dirty);
      driver->card(5, testDriver::card_state_younger);
      driver->card(6, testDriver::card_state_younger);

      server->stop_compensation_timer();

      if (server->is_connected(event)) {
	server->transmit_streams();
      }
    }
    server->safepoint(event);

    ++counter;
  }

  return NULL;
}
}

int
main (int argc, char *argv[]) {
  int res;
#ifdef _SOLARIS_
  thread_t tid;
#endif //_SOLARIS_
#ifdef _LINUX_
  pthread_t tid;
#endif //_LINUX_
  int port;

  if (argc != 2) {
    gcspy_raise_error("Wrong number of arguments");
  }
  port = atoi(argv[1]);

  printf("-- Initialising main server on port %d\n", port);
  
  server = new gcspyMainServer(port, "Test Server", MAX_LEN, 1);
  server->add_event(EVENT_ONE, "Event One");
  server->add_event(EVENT_TWO, "Event Two");

  driver = new testDriver("Old Generation", 100, 1);
  server->add_space(driver);

#ifdef _SOLARIS_
  res = thr_create(NULL, 0, main_loop, NULL, THR_BOUND, &tid);
#endif //_SOLARIS_

#ifdef _LINUX_
  res = pthread_create(&tid, NULL, main_loop, NULL);
#endif //_LINUX_

  if (res != 0) {
    printf("Couldn't create thread.\n");
    exit(-1);
  }

  server->main_loop();
}
