package gcspy.vis;

import gcspy.interpreter.client.EventCommandListener;

import java.awt.BorderLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Slider for events that includes an actual {@link JSlider} and a
 * {@link JLabel} indicating the current position (index). Both can be used to
 * jump to specific events directly.
 * 
 * @author Thomas Schatzl
 * @author Peter Hofer
 */
public class EventSlider extends JComponent implements EventCommandListener {

	private int eventCount;

	private JFormattedTextField value;

	private JSlider slider;

	private List<EventSliderMovedListener> listeners = new ArrayList<EventSliderMovedListener>();

	public EventSlider() {
		setLayout(new BorderLayout());

		slider = new JSlider(JSlider.HORIZONTAL);
		slider.setPaintTicks(true);
		slider.setMajorTickSpacing(200);
		slider.setMinorTickSpacing(50);
		slider.setToolTipText("Move to event");
		slider.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				if (!slider.getValueIsAdjusting()) {
					value.setText(Integer.toString(slider.getValue()));
					fireMovedListeners();
				}
			}
		});
		add(slider, BorderLayout.CENTER);

		value = new JFormattedTextField(NumberFormat.getIntegerInstance());
		value.setColumns(5);
		value.setHorizontalAlignment(JFormattedTextField.CENTER);
		value.addPropertyChangeListener("value", new PropertyChangeListener() {
			@Override
			public void propertyChange(PropertyChangeEvent evt) {
				slider.setValue(Integer.parseInt(value.getText()));
			}
		});
		add(value, BorderLayout.EAST);

		reset();
	}
	
	public void reset() {
		eventCount = 0;
		slider.setValue(0);
		slider.setMinimum(0);
		slider.setMaximum(0);
	}

	public void addMovedListener(EventSliderMovedListener listener) {
		if (listener == null)
			throw new IllegalArgumentException("listener");
		listeners.add(listener);
	}

	public void removeMovedListener(EventSliderMovedListener listener) {
		listeners.remove(listener);
	}

	private void fireMovedListeners() {
		for (EventSliderMovedListener listener : listeners) {
			listener.eventSliderMoved(slider.getValue());
		}
	}

	public void setEnabled(boolean enabled) {
		boolean oldEnabled = isEnabled();
		super.setEnabled(enabled);
		if (oldEnabled != enabled) {
			slider.setEnabled(enabled);
			value.setEnabled(enabled);
		}
	}

	/**
	 * Move the slider to the specified event.
	 * 
	 * @param event
	 *            The event to set the slider to.
	 */
	public void moveSlider(int event) {
		slider.setValue(event);
		// Event handler will do the rest
	}

	@Override
	public void onEventCmd(int eventID, int elapsedTime, int compensationTime) {
		eventCount++;
		slider.setMinimum(0);
		slider.setMaximum(eventCount - 1);
		slider.setValue(eventCount - 1);
	}
}
