/**
 ** $Id: HistoryConfigDialog.java 28 2005-06-20 13:13:35Z rej $
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

package gcspy.vis.plugins.history;

import gcspy.interpreter.Space;
import gcspy.vis.InterpreterEventProvider;
import gcspy.vis.TileStyle;
import gcspy.vis.utils.ColorChooser;
import gcspy.vis.utils.Factory;
import gcspy.vis.utils.OKCancelDialog;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Dialog for the history graph settings
 * 
 * @author Tony Printezis
 */
class HistoryConfigDialog extends OKCancelDialog {

	private HistoryConfig config;

	private JTextField colsTF;
	private JTextField rowsTF;
	private JTextField tileSizeTF;
	private JTextField imageSizeTF;
	private JComboBox streamCB;
	private JCheckBox horSepCB;
	private JComboBox horSepEventCB;
	private JCheckBox verSepCB;
	private JTextField verSepPeriodTF;
	private JCheckBox verSepLegendCB;
	private JTextField verSepLegendPeriodTF;
	private JTextField verSepLegendStringPostTF;
	private JCheckBox spaceSepCB;
	private ColorChooser spaceSepCC;
	private ColorChooser unusedCC;
	private ColorChooser zeroCC;
	private ColorChooser lowCC;
	private ColorChooser hiCC;
	private ColorChooser horSepCC;
	private ColorChooser verSepCC;
	private ColorChooser bgCC;
	private ColorChooser borderCC;
	private ColorChooser titleCC;
	private JTextField fontSizeTF;
	private JTextField borderSizeTF;
	private JTextField titlePreTF;
	private JCheckBox titleCB;

	private TileStyle tmColors;

	private JButton validateB;
	private JButton defaultsB;

	private int noStreamIndex;

	private class DefaultsBListener implements ActionListener {
		private Component component;

		public void actionPerformed(ActionEvent event) {
			config.revertToDefaults(component, tmColors);
			updateFromConfig();
		}

		DefaultsBListener(Component component) {
			this.component = component;
		}
	}

	private class ValidateBListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			updateConfig();
			updateFromConfig();
		}
	}

	private class VerSepPanelListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			updateVerSepPanel();
		}
	}

	private class HorSepPanelListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			updateHorSepPanel();
		}
	}

	private class SpaceSepPanelListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			updateSpaceSepPanel();
		}
	}

	private class TitlePanelListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			updateTitlePanel();
		}
	}

	private class StreamCBListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			if (hiCC != null) {
				// int stream = streamCB.getSelectedIndex();
				// Color color = tmColors.getStreamColor(stream);
				Color color = getCurrentStreamColor();
				hiCC.setColor(color);
			}
		}
	}

	private Color getCurrentStreamColor() {
		int stream = streamCB.getSelectedIndex();
		Color color;
		if (stream == noStreamIndex) {
			color = Color.BLACK;
		} else
			color = tmColors.getStreamColor(stream);
		return color;
	}

	/** The OK button has been clicked */
	protected void onOkClicked() {
		updateConfig();
		super.onOkClicked();
	}

	private void updateVerSepPanel() {
		boolean selected = verSepCB.isSelected();
		verSepPeriodTF.setEnabled(selected);
		verSepCC.setEnabled(selected);
		verSepLegendCB.setEnabled(selected);

		boolean legendselected = (selected && verSepLegendCB.isSelected());
		verSepLegendPeriodTF.setEnabled(legendselected);
		verSepLegendStringPostTF.setEnabled(legendselected);
	}

	private void updateHorSepPanel() {
		boolean selected = horSepCB.isSelected();
		horSepEventCB.setEnabled(selected);
		horSepCC.setEnabled(selected);
	}

	private void updateSpaceSepPanel() {
		boolean selected = spaceSepCB.isSelected();
		if (selected) {
			spaceSepCC.setEnabled(selected);
		} else {
			spaceSepCC.setEnabled(selected);
		}
	}

	private void updateTitlePanel() {
		boolean selected = titleCB.isSelected();
		titleCC.setEnabled(selected);
		titlePreTF.setEnabled(selected);
	}

	private void updateFromConfig() {
		int width = config.calcWidth();
		int height = config.calcHeight();
		String imageSize = width + "x" + height;

		colsTF.setText("" + config.getCols());
		rowsTF.setText("" + config.getRows());
		tileSizeTF.setText("" + config.getTileSize());
		imageSizeTF.setText(imageSize);
		int stream = config.getStream();
		if (stream == -1)
			streamCB.setSelectedIndex(noStreamIndex);
		else
			streamCB.setSelectedIndex(stream);

		horSepEventCB.setSelectedIndex(config.getHorSepEvent());
		horSepCB.setSelected(config.horSep());
		updateHorSepPanel();

		verSepCB.setSelected(config.verSep());
		verSepPeriodTF.setText("" + config.getVerSepPeriod());
		verSepLegendCB.setSelected(config.getVerSepLegend());
		verSepLegendPeriodTF.setText("" + config.getVerSepLegendPeriod());
		verSepLegendStringPostTF.setText(""
				+ config.getVerSepLegendStringPost());
		updateVerSepPanel();

		spaceSepCB.setSelected(config.getSpaceSep());
		spaceSepCC.setColor(config.getSpaceSepColor());
		updateSpaceSepPanel();

		unusedCC.setColor(config.getUnusedColor());
		zeroCC.setColor(config.getZeroColor());
		lowCC.setColor(config.getLowColor());
		hiCC.setColor(config.getHiColor());
		horSepCC.setColor(config.getHorSepColor());
		verSepCC.setColor(config.getVerSepColor());
		bgCC.setColor(config.getBGColor());

		titleCB.setSelected(config.getTitle());
		titleCC.setColor(config.getTitleColor());
		titlePreTF.setText(config.getTitlePre());
		updateTitlePanel();

		borderSizeTF.setText("" + config.getBorderSize());
		borderCC.setColor(config.getBorderColor());

		fontSizeTF.setText("" + config.getFontSize());
	}

	private void updateConfig() {
		int stream = streamCB.getSelectedIndex();
		if (stream == noStreamIndex)
			stream = -1;
		config = new HistoryConfig(this, config.getCols(), HistoryConfig
				.parseRows(rowsTF.getText()), HistoryConfig
				.parseTileSize(tileSizeTF.getText()), stream, horSepCB
				.isSelected(), horSepEventCB.getSelectedIndex(), verSepCB
				.isSelected(), HistoryConfig.parseVerSepPeriod(verSepPeriodTF
				.getText()), verSepLegendCB.isSelected(), HistoryConfig
				.parseVerSepLegendPeriod(verSepLegendPeriodTF.getText()),
				verSepLegendStringPostTF.getText(), spaceSepCB.isSelected(),
				spaceSepCC.getColor(), unusedCC.getColor(), zeroCC.getColor(),
				lowCC.getColor(), hiCC.getColor(), horSepCC.getColor(),
				verSepCC.getColor(), bgCC.getColor(), titleCB.isSelected(),
				titlePreTF.getText(), titleCC.getColor(), HistoryConfig
						.parseBorderSize(borderSizeTF.getText()), borderCC
						.getColor(), HistoryConfig.parseFontSize(fontSizeTF
						.getText()));
	}

	private void setup(HistoryConfig config, InterpreterEventProvider interpreter,
			Space space, TileStyle tmColors) {
		int numStreams = space.getStreamNum();

		this.config = config;
		this.tmColors = tmColors;
		this.noStreamIndex = numStreams;

		JPanel panel;
		Container cont = getContentPane();

		Container generalPanel = new Box(BoxLayout.Y_AXIS);
		Container horSepPanel = new Box(BoxLayout.Y_AXIS);
		Container colorPanel = new Box(BoxLayout.Y_AXIS);
		Container verSepPanel = new Box(BoxLayout.Y_AXIS);
		Container spaceSepPanel = new Box(BoxLayout.Y_AXIS);
		Container titlePanel = new Box(BoxLayout.Y_AXIS);
		Container borderPanel = new Box(BoxLayout.Y_AXIS);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Columns:"));
		colsTF = Factory.createTextField("", 6, false);
		panel.add(colsTF);
		generalPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Max Rows:"));
		rowsTF = Factory.createTextField("", 6, true);
		panel.add(rowsTF);
		generalPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Tile Size:"));
		tileSizeTF = Factory.createTextField("", 3, true);
		panel.add(tileSizeTF);
		generalPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Image Size:"));
		imageSizeTF = Factory.createTextField("", 10, false);
		panel.add(imageSizeTF);
		generalPanel.add(panel);

		hiCC = null;
		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Stream:"));
		streamCB = Factory.createComboBox(new StreamCBListener());
		for (int i = 0; i < numStreams; ++i)
			streamCB.addItem(space.getStream(i).getName());
		streamCB.addItem("<none>");
		panel.add(streamCB);
		generalPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Enabled:"));
		horSepCB = Factory.createCheckBox(null, false,
				new HorSepPanelListener());
		panel.add(horSepCB);
		horSepPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Trigger Event:"));
		horSepEventCB = Factory.createComboBox();
		for (String s : interpreter.getEvents().getNames()) {
			horSepEventCB.addItem(s);
		}
		panel.add(horSepEventCB);
		horSepPanel.add(panel);

		VerSepPanelListener listener = new VerSepPanelListener();
		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Enabled:"));
		verSepCB = Factory.createCheckBox(null, false, listener);
		panel.add(verSepCB);
		verSepPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Period:"));
		verSepPeriodTF = Factory.createTextField("", 4, true);
		panel.add(verSepPeriodTF);
		verSepPanel.add(panel);

		panel = Factory.createGridBagPanel();
		verSepCC = new ColorChooser();
		verSepCC.add(Factory.createLabel("Color:"));
		verSepPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Legend:"));
		verSepLegendCB = Factory.createCheckBox(null, false, listener);
		panel.add(verSepLegendCB);
		verSepPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Legend Period:"));
		verSepLegendPeriodTF = Factory.createTextField("", 6, true);
		panel.add(verSepLegendPeriodTF);
		verSepPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Legend Postfix:"));
		verSepLegendStringPostTF = Factory.createTextField("", 6, true);
		panel.add(verSepLegendStringPostTF);
		verSepPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Enabled:"));
		spaceSepCB = Factory.createCheckBox(null, false,
				new SpaceSepPanelListener());
		panel.add(spaceSepCB);
		spaceSepPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Color:"));
		spaceSepCC = new ColorChooser();
		panel.add(spaceSepCC);
		spaceSepPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Zero Color:"));
		zeroCC = new ColorChooser();
		panel.add(zeroCC);
		colorPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Low Color:"));
		lowCC = new ColorChooser();
		panel.add(lowCC);
		colorPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("High Color:"));
		hiCC = new ColorChooser();
		panel.add(hiCC);
		colorPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Unused Color:"));
		unusedCC = new ColorChooser();
		panel.add(unusedCC);
		colorPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Background Color:"));
		bgCC = new ColorChooser();
		panel.add(bgCC);
		colorPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Color:"));
		horSepCC = new ColorChooser();
		panel.add(horSepCC);
		horSepPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Font Size:"));
		fontSizeTF = Factory.createTextField("", 3, true);
		panel.add(fontSizeTF);
		generalPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Enabled:"));
		titleCB = Factory.createCheckBox(null, false, new TitlePanelListener());
		panel.add(titleCB);
		titlePanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Color:"));
		titleCC = new ColorChooser();
		panel.add(titleCC);
		titlePanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Title Prefix:"));
		titlePreTF = Factory.createTextField("", 16, true);
		panel.add(titlePreTF);
		titlePanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Border Size:"));
		borderSizeTF = Factory.createTextField("", 3, true);
		panel.add(borderSizeTF);
		borderPanel.add(panel);

		panel = Factory.createFlowPanel(FlowLayout.LEFT);
		panel.add(Factory.createLabel("Color:"));
		borderCC = new ColorChooser();
		panel.add(borderCC);
		borderPanel.add(panel);

		Container left = new Box(BoxLayout.Y_AXIS);
		Container right = new Box(BoxLayout.Y_AXIS);

		left.add(Factory.createTitlePanel("General", generalPanel));
		left.add(Factory.createTitlePanel("Vertical Separators", verSepPanel));
		left.add(Factory.createTitlePanel("Title", titlePanel));
		JPanel outerLeft = Factory.createBorderPanel();
		outerLeft.add(left, BorderLayout.NORTH);

		right.add(Factory.createTitlePanel("Colors", colorPanel));
		right.add(Factory
				.createTitlePanel("Horizontal Separators", horSepPanel));
		right.add(Factory.createTitlePanel("Space Separators", spaceSepPanel));
		right.add(Factory.createTitlePanel("Border", borderPanel));
		JPanel outerRight = Factory.createBorderPanel();
		outerRight.add(right, BorderLayout.NORTH);

		JPanel grid = Factory.createGridPanel(1, 2);
		grid.add(outerLeft);
		grid.add(outerRight);

		panel = Factory.createTitlePanel("History Graph Configuration", grid);
		cont.add(panel, BorderLayout.CENTER);
		
		JPanel buttonPanel = super.getButtonPanel();
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3)); // padding
		defaultsB = Factory.createButton("Defaults", true,
			new DefaultsBListener(this));
		buttonPanel.add(defaultsB);
		validateB = Factory.createButton("Validate", true,
			new ValidateBListener());
		buttonPanel.add(validateB);
		cont.add(buttonPanel, BorderLayout.SOUTH);

		pack();

		updateFromConfig();
	}

	/**
	 * Get the configuration for the history view
	 * 
	 * @return the configuration
	 */
	HistoryConfig getConfig() {
		return config;
	}

	/**
	 * A new configuration dialog for the history view
	 * 
	 * @param owner
	 *            The parent frame
	 * @param config
	 *            The history configuration
	 * @param interpreter
	 *            The space's interpreter
	 * @param space
	 *            The space
	 * @param tmColors
	 *            The tile colour configuration
	 */
	HistoryConfigDialog(JFrame owner, HistoryConfig config,
			InterpreterEventProvider interpreter, Space space, TileStyle tmColors) {
		super(owner);
		setup(config, interpreter, space, tmColors);
	}

}
